﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
//額外引用
using NorthwindPaging.Models;           //自訂 Order 資料表
using Microsoft.AspNet.Identity.Owin;   //ApplicationDbContext、Get()
using DataTables.Mvc;       //DataTablesBinder、IDataTablesRequest、DataTablesResponse
using System.Linq.Dynamic;  //OrderBy()

namespace NorthwindPaging.Controllers
{
    public class OrderController : Controller
    {
        private ApplicationDbContext _dbContext;

        public ApplicationDbContext DbContext
        {
            get
            {
                return _dbContext ?? HttpContext.GetOwinContext().Get<ApplicationDbContext>();  //資料庫驗證、取得連線
            }
            private set
            {
                _dbContext = value;
            }
        }

        public OrderController(ApplicationDbContext dbContext)
        {
            _dbContext = dbContext;
        }

        public OrderController()
        {
        }

        // GET: Order
        //public ActionResult Index()
        //{
        //    return View();
        //}

        public ActionResult OrderGridView()
        {
            return View();
        }

        public ActionResult Get([ModelBinder(typeof(DataTablesBinder))] IDataTablesRequest requestModel)
        {
            IQueryable<Order> query = DbContext.Orders; //LINQ to Entites
            var totalCount = query.Count();     //Order 資料表全部 830 筆

            #region Filtering (使用者輸入關鍵字搜尋)
            // Apply filters for searching (DataTables.Mvc.Search 類別)
            if (requestModel.Search.Value != string.Empty)
            {
                var value = requestModel.Search.Value.Trim();

                //filtering 功能 (欄位必須要 string，欄位 int、decimal 預設不行，因此加上ToString())
                query = query.Where(o => o.OrderID.ToString().Contains(value) ||
                                         o.CustomerID.Contains(value) ||
                                         o.EmployeeID.ToString().Contains(value) || 
                                         o.ShipCountry.Contains(value) ||
                                         o.Freight.ToString().Contains(value)                                         
                                   );
            }

            var filteredCount = query.Count();  //全部 830 筆

            #endregion Filtering

            #region Sorting
                        
            var sortedColumns = requestModel.Columns.GetSortedColumns();
            var orderByString = String.Empty;

            foreach (var column in sortedColumns)
            {
                orderByString += orderByString != String.Empty ? "," : "";
                orderByString += (column.Data) + (column.SortDirection == Column.OrderDirection.Ascendant ? " asc" : " desc");
            }
            //Dynamic LINQ
            query = query.OrderBy(orderByString == string.Empty ? "OrderID asc" : orderByString);    //預設的排序欄位、排序方式

            #endregion Sorting

            #region Paging
            query = query.Skip(requestModel.Start).Take(requestModel.Length);   //Skip:起始index，Take:要撈的筆數(使用者可由下拉選單選擇)

            int i1 = query.Count(); //已用 SQL Server 的 OFFSET-FETCH 或 ROW_NUMBER()，過濾出畫面上該頁要顯示的 10 筆

            //用 Select() 重新組裝我們需要的資料
            var data = query.Select(order => new
            {
                OrderID = order.OrderID, 
                CustomerID = order.CustomerID,
                EmployeeID = order.EmployeeID, 
                ShipCountry = order.ShipCountry, 
                Freight = order.Freight
            }).ToList();

            #endregion Paging

            int i2 = query.Count(); //10
            int i3 = data.Count();  //10
            int i4 = requestModel.Draw;     //1、2、...(非頁數，第1頁為1，但點擊第3頁、第4頁皆為2)
            int i5 = requestModel.Start;    //起始index，第1頁為0、第2頁為10、第4頁為30、...
            int i6 = requestModel.Length;   //10. tells how many records per page user wants to see which is also configurable by user using combo box on the page
            int i7 = filteredCount; // 830 筆
            int i8 = totalCount;    // 830 筆

            //參數1:datatables.mvc5 的自訂類別 DataTablesResponse， 參數2:允許 HTTP GET 存取
            return Json(new DataTablesResponse(requestModel.Draw, data, filteredCount, totalCount), JsonRequestBehavior.AllowGet);
        }
    }
}